# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import repl_pb2 as repl__pb2


class ReplStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

    Args:
      channel: A grpc.Channel.
    """
        self.Run = channel.unary_unary(
            '/coop.rchain.node.model.Repl/Run',
            request_serializer=repl__pb2.CmdRequest.SerializeToString,
            response_deserializer=repl__pb2.ReplResponse.FromString,
        )
        self.Eval = channel.unary_unary(
            '/coop.rchain.node.model.Repl/Eval',
            request_serializer=repl__pb2.EvalRequest.SerializeToString,
            response_deserializer=repl__pb2.ReplResponse.FromString,
        )


class ReplServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def Run(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Eval(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ReplServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Run': grpc.unary_unary_rpc_method_handler(
            servicer.Run,
            request_deserializer=repl__pb2.CmdRequest.FromString,
            response_serializer=repl__pb2.ReplResponse.SerializeToString,
        ),
        'Eval': grpc.unary_unary_rpc_method_handler(
            servicer.Eval,
            request_deserializer=repl__pb2.EvalRequest.FromString,
            response_serializer=repl__pb2.ReplResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'coop.rchain.node.model.Repl', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))



from unittest import mock

import pytest

from rchain_grpc import utils
from rchain_grpc.exceptions import ConnectionClosedException


@pytest.fixture
def stub():
    s = mock.MagicMock()
    s.test_method.return_value = 22
    return s


@pytest.fixture
def channel():
    return mock.MagicMock()


@pytest.fixture
def connection(channel, stub):
    return utils.create_connection(lambda _: stub, channel_fn=lambda _: channel)


def test_connection_works_as_proxy_to_stub(channel, stub, connection):
    args = [1, '2']
    value = 'test value'

    ret = connection.test_method(*args)
    assert ret == 22
    stub.test_method.assert_called_once_with(*args)

    connection.value = value
    assert stub.value == value


def test_connection_as_contextmanager(channel, stub, connection):
    with connection as connection_from_context_manager:
        ret = connection_from_context_manager.test_method(1)
        assert ret == 22
        assert isinstance(connection_from_context_manager, utils.Connection)
        assert connection_from_context_manager is connection
    with pytest.raises(ConnectionClosedException):
        connection.test_method(1)


@pytest.mark.parametrize(
    "d1,d2,result",
    [
        ({}, {}, True),
        ({'a': 1}, {'a': 1}, True),
        ({'a': 1}, {'a': 2}, False),
        ({'a': {'b': 1}}, {'a': {}}, False),
        ({'a': [1], 'b': 11}, {'b': 11, 'a': [1]}, True),
    ],
)
def test_is_equal(d1, d2, result):
    assert utils.is_equal(d1, d2) == result



import functools
import json
from contextlib import AbstractContextManager
from typing import Any, Callable, Generic, Type, TypeVar

from grpc import Channel, insecure_channel

from .exceptions import ConnectionClosedException

gRPCStub = TypeVar('gRPCStub')


class Connection(AbstractContextManager, Generic[gRPCStub]):
    """proxy class from stub from grpc. Support `close` method
    and can be usee as context manager"""

    def __init__(self, channel: Channel, stub: gRPCStub) -> None:
        # using super for avoiding recursion
        super(Connection, self).__setattr__('connected', True)
        super(Connection, self).__setattr__('_channel', channel)
        super(Connection, self).__setattr__('_stub', stub)

    def close(self) -> None:
        self._channel.close()
        del self._channel
        del self._stub
        # using super for avoiding recursion
        super(Connection, self).__setattr__('connected', False)

    def __getattr__(self, key: str) -> Any:
        if not self.connected:
            raise ConnectionClosedException()
        return getattr(self._stub, key)

    def __setattr__(self, key: str, val: Any) -> None:
        if not self.connected:
            raise ConnectionClosedException()
        setattr(self._stub, key, val)

    def __exit__(self, *args, **kwargs) -> None:
        self.close()
        return super(Connection, self).__exit__(*args, **kwargs)


def create_connection(
    stub: Type[gRPCStub],
    channel_fn: Callable[[str], Channel] = insecure_channel,
    host: str = '127.0.0.1',
    port: int = 40401,
) -> Connection[gRPCStub]:
    """create new connection from `stub` generated by grpcio.
    Connection can be used as context manager"""

    channel = channel_fn(f'{host}:{port}')
    return Connection[stub](channel, stub(channel))


def create_connection_builder(stub: Type[Connection]) -> Callable[..., Connection]:
    """factory for cinnection functions created from `stub` generated by grpcio"""

    return functools.partial(create_connection, stub=stub)


def is_equal(d1: dict, d2: dict) -> bool:
    kwargs = {'sort_keys': True, 'indent': 0}
    return json.dumps(d1, **kwargs) == json.dumps(d2, **kwargs)



import random
import secrets
from concurrent import futures

import pytest
from rchain_grpc import casper, exceptions


@pytest.fixture
def connection(rchain_host):
    with casper.create_connection(host=rchain_host) as conn:
        yield conn


def test_get_blocks(connection):
    blocks = casper.get_blocks(connection)
    fields = {
        'blockHash',
        'blockSize',
        'deployCount',
        'tupleSpaceHash',
        'tupleSpaceDump',
        'timestamp',
        'faultTolerance',
        'shardId',
    }
    assert isinstance(blocks, list)
    assert len(blocks) > 0
    for block in blocks:
        assert fields.issubset(set(block.keys()))


@pytest.fixture
def rchain_ch_name():
    sufix = secrets.token_hex(5)
    return f'ch_name_{sufix}'


@pytest.fixture
def rchain_ch_value():
    return secrets.token_hex(5)


@pytest.fixture
def deployed(connection, rchain_ch_name, rchain_ch_value):
    rho_term = f'@"{rchain_ch_name}"!("{rchain_ch_value}")'
    return casper.deploy(connection, rho_term)


@pytest.fixture
def proposed(connection, deployed):
    return casper.propose(connection)


def test_deploy(deployed):
    assert deployed == {'message': 'Success!', 'success': True}


def test_propose(proposed):
    assert isinstance(proposed, dict)
    assert proposed['success'] == True
    assert 'created and added' in proposed['message']


def test_get_value_from(proposed, rchain_ch_name, rchain_ch_value, connection):
    ret = casper.get_value_from(connection, rchain_ch_name)
    # TODO: test with channels with more data and figure out how to remove
    #       this nested list from here
    assert ret['blockResults'][0]['postBlockData'] == [[rchain_ch_value]]


def test_get_value_from_empty_channel(connection, rchain_ch_name):
    ret = casper.get_value_from(connection, f'not-{rchain_ch_name}')
    assert ret == None


def test_run_and_get_value_from(connection, rchain_ch_value):
    term = f'proof_output!("{rchain_ch_value}")'
    ret = casper.run_and_get_value_from(connection, term)
    assert ret['blockResults'][0]['postBlockData'] == [[rchain_ch_value]]


def test_listen_on(deployed, connection, rchain_ch_name, rchain_ch_value):
    def run():
        return next(casper.listen_on(connection, rchain_ch_name))

    with futures.ThreadPoolExecutor() as executor:
        future = executor.submit(run)
        proposed(connection, deployed)
        assert future.result(timeout=5)['blockResults'][0]['postBlockData'] == [
            [rchain_ch_value]
        ]


def test_listen_on_timeout_if_not_deployed_and_proposed(connection, rchain_ch_name):
    with pytest.raises(exceptions.TimeoutException) as ex:
        next(casper.listen_on(connection, rchain_ch_name, timeout=1))


@pytest.mark.parametrize(
    'args,expected',
    [
        ('true', [[True]]),
        ('false', [[False]]),
        ('1, false', [[1], [False]]),
        ('{}.set("x", 24).set("y", "value")', [[{'x': 24, 'y': 'value'}]]),
    ],
)
def test_value_conversion(args, expected, connection):
    term = f'proof_output!({args})'
    ret = casper.run_and_get_value_from(connection, term)
    assert ret['blockResults'][0]['postBlockData'] == expected


@pytest.mark.parametrize("contract", ["add1", ["add", "1"]])
def test_run_contract(contract, connection, add_contract_path):
    number = random.randint(100, 1000)
    with open(add_contract_path) as contract_file:
        term = contract_file.read()
    casper.deploy(connection, term)
    casper.propose(connection)
    result = casper.run_contract(connection, contract, [number])
    assert result['blockResults'][0]['postBlockData'][0][0] == number + 1
